import math
import turtle


scale = 30

class Point:
    def __init__(self, x: int, y: int, label: int):
        self.x = x
        self.y = y
        self.label = label
    
    def __repr__(self) -> str:
        return str(self)
    
    def __str__(self) -> str:
        return f"P{self.label}({self.x}, {self.y})"
    
    def as_tuple(self) -> tuple:
        return (self.x, self.y)
    
    def __eq__(self, other: "Point") -> bool:
        return self.x == other.x and self.y == other.y

class Vector:
    def __init__(self, start: Point, end: Point):
        self.start = start
        self.end = end
        self.length = math.sqrt((self.start.y - self.end.y) ** 2 + (self.start.x - self.end.x) ** 2)
    
    def angle_to(self, other: "Vector") -> float:
        points = (self.end.as_tuple(), other.end.as_tuple(), self.start.as_tuple(), other.start.as_tuple())
        common_point = next(point for point in points if points.count(point) > 1)
        uncommon_points = [Point(*point, 0) for point in points if point != common_point]
        cos = (self.length ** 2 + other.length ** 2 - Vector(uncommon_points[0], uncommon_points[1]).length ** 2) / (2 * self.length * other.length)
        angle = math.acos(cos) * 180 / math.pi
        
        base_k = (other.end.y - other.start.y) / (other.end.x - other.start.x) * (-1 if self.start.x > self.end.x else 1)
        this_k = (self.end.y - self.start.y) / (self.end.x - self.start.x) * (-1 if self.start.x > self.end.x else 1)

        if this_k - base_k == 0:
            return 180

        return angle + (180 if this_k - base_k < 0 else 0)
    
    def __repr__(self) -> str:
        return str(self)
    
    def __str__(self) -> str:
        return f"Vector({self.start} -> {self.end})"



points = [Point(-1, -1, 1), Point(1, -2, 2), Point(4, -2, 3), Point(2, -4, 4), Point(2, 3, 5), Point(-4, -5, 6)]


def get_vectors(points: list[Point], start_index: int, following_index: int) -> list[Vector]:
    """
    Forms up vectors and sorts them by angle to base vector (start -> following)
    :param points:
    :param start_index:
    :param following_index:
    """
    start, following = points[start_index], points[following_index]
    vectors = [Vector(following, i) for j, i in enumerate(points) if j not in {following_index}]
    base = Vector(start, following)

    vectors.sort(key=lambda x: x.angle_to(base))
    print([(str(vector), vector.angle_to(base)) for vector in vectors])
    return vectors


def get_ends(vectors: list[Vector]) -> list[Point]:
    return [i.end for i in vectors]


def get_next_points(points: list[Point], start: int, following: int, t: int) -> tuple[Point, Point]:
    vectors = get_vectors(points, start, following)
    visualize(vectors)
    new_points = get_ends(vectors)
    start, following = new_points[t], points[start]
    return (start, following)


def visualize(vectors: list[Vector]):
    for vector in vectors:
        turtle.penup()
        turtle.goto((vector.start.x * scale, vector.start.y * scale))
        turtle.pendown()
        turtle.goto((vector.end.x * scale, vector.end.y * scale))
